/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.items.tool;

import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.class_1309;
import net.minecraft.class_1743;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1800;
import net.minecraft.class_1834;
import net.minecraft.class_1890;
import net.minecraft.class_1893;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_2680;
import net.minecraft.class_2960;
import net.minecraft.class_3614;
import net.minecraft.item.*;
import reborncore.api.power.IEnergyItemInfo;
import reborncore.api.power.ItemPowerManager;
import reborncore.common.powerSystem.ExternalPowerSystems;
import reborncore.common.powerSystem.PowerSystem;
import reborncore.common.util.ItemDurabilityExtensions;
import reborncore.common.util.ItemUtils;
import techreborn.TechReborn;

import javax.annotation.Nullable;
import java.util.Random;

public class ItemChainsaw extends class_1743 implements IEnergyItemInfo, ItemDurabilityExtensions {

	public int maxCharge = 1;
	public int cost = 250;
	public float poweredSpeed = 20F;
	public float unpoweredSpeed = 2.0F;
	public int transferLimit = 100;
	public boolean isBreaking = false;

	public ItemChainsaw(class_1834 material, int energyCapacity, float unpoweredSpeed) {
		super(material, (int) material.method_8028(), unpoweredSpeed, new class_1792.class_1793().method_7892(TechReborn.ITEMGROUP).method_7889(1));
		this.maxCharge = energyCapacity;

		this.method_7863(new class_2960("techreborn", "animated"), new class_1800() {
			@Override
			@Environment(EnvType.CLIENT)
			public float call(class_1799 stack, @Nullable class_1937 worldIn, @Nullable class_1309 entityIn) {
				if (!stack.method_7960() && new ItemPowerManager(stack).getEnergyStored() >= cost
						&& entityIn != null && entityIn.method_6047().equals(stack)) {
					return 1.0F;
				}
				return 0.0F;
			}
		});
	}

	// ItemAxe
	@Override
	public float method_7865(class_1799 stack, class_2680 state) {
		if (new ItemPowerManager(stack).getEnergyStored() >= cost
				&& (state.method_11620() == class_3614.field_15932)) {
			return this.poweredSpeed;
		} else {
			return super.method_7865(stack, state);
		}
	}

	// ItemTool
	@Override
	public boolean method_7879(class_1799 stack, class_1937 worldIn, class_2680 blockIn, class_2338 pos, class_1309 entityLiving) {
		Random rand = new Random();
		if (rand.nextInt(class_1890.method_8225(class_1893.field_9119, stack) + 1) == 0) {
			ItemPowerManager capEnergy = new ItemPowerManager(stack);

			capEnergy.useEnergy(cost, false);
			ExternalPowerSystems.requestEnergyFromArmor(capEnergy, entityLiving);
		}
		return true;
	}

	@Override
	public boolean method_7873(class_1799 itemstack, class_1309 entityliving, class_1309 entityliving1) {
		return true;
	}

	@Override
	public double getDurability(class_1799 stack) {
		return 1 - ItemUtils.getPowerForDurabilityBar(stack);
	}

	@Override
	public boolean showDurability(class_1799 stack) {
		return true;
	}

	@Override
	public int getDurabilityColor(class_1799 stack) {
		return PowerSystem.getDisplayPower().colour;
	}

	@Override
	public boolean method_7878(class_1799 itemStack_1, class_1799 itemStack_2) {
		return false;
	}

	// IEnergyItemInfo
	@Override
	public int getCapacity() {
		return maxCharge;
	}

	@Override
	public int getMaxInput() {
		return transferLimit;
	}

	@Override
	public int getMaxOutput() {
		return 0;
	}

	@Override
	public int method_7837() {
		return 20;
	}
}
