/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.items.tool.industrial;

import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.class_1074;
import net.minecraft.class_124;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1271;
import net.minecraft.class_1309;
import net.minecraft.class_1657;
import net.minecraft.class_1761;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1834;
import net.minecraft.class_1836;
import net.minecraft.class_1937;
import net.minecraft.class_2246;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2371;
import net.minecraft.class_239;
import net.minecraft.class_2487;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2680;
import net.minecraft.class_3614;
import net.minecraft.class_3959;
import net.minecraft.class_3965;
import net.minecraft.util.*;
import reborncore.api.power.ItemPowerManager;
import reborncore.common.powerSystem.ExternalPowerSystems;
import reborncore.common.util.ChatUtils;
import reborncore.common.util.ItemUtils;
import techreborn.config.TechRebornConfig;
import techreborn.init.TRContent;
import techreborn.items.tool.ItemDrill;
import techreborn.utils.InitUtils;
import techreborn.utils.MessageIDs;

import javax.annotation.Nullable;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class ItemIndustrialDrill extends ItemDrill {

	// 4M FE max charge with 1k charge rate
	public ItemIndustrialDrill() {
		super(class_1834.field_8930, TechRebornConfig.IndustrialDrillCharge, 2.0F, 10F);
		this.cost = 250;
		this.transferLimit = 1000;
	}

	private Set<class_2338> getTargetBlocks(class_1937 worldIn, class_2338 pos, @Nullable class_1657 playerIn) {
		Set<class_2338> targetBlocks = new HashSet<class_2338>();
		if (playerIn == null) {
			return new HashSet<>();
		}

		//Put a dirt block down to raytrace with to stop it raytracing past the intended block
		worldIn.method_8501(pos, class_2246.field_10566.method_9564());
		class_239 hitResult = method_7872(worldIn, playerIn, class_3959.class_242.field_1348);
		worldIn.method_8501(pos, class_2246.field_10124.method_9564());

		if(!(hitResult instanceof class_3965)){
			return Collections.emptySet();
		}
		class_2350 enumfacing = ((class_3965) hitResult).method_17780();
		if (enumfacing == class_2350.field_11035 || enumfacing == class_2350.field_11043) {
			for (int i = -1; i < 2; i++) {
				for (int j = -1; j < 2; j++) {
					class_2338 newPos = pos.method_10069(i, j, 0);
					if (shouldBreak(playerIn, worldIn, pos, newPos)) {
						targetBlocks.add(newPos);
					}
				}
			}
		} else if (enumfacing == class_2350.field_11034 || enumfacing == class_2350.field_11039) {
			for (int i = -1; i < 2; i++) {
				for (int j = -1; j < 2; j++) {
					class_2338 newPos = pos.method_10069(0, j, i);
					if (shouldBreak(playerIn, worldIn, pos, newPos)) {
						targetBlocks.add(newPos);
					}
				}
			}
		} else if (enumfacing == class_2350.field_11033 || enumfacing == class_2350.field_11036) {
			for (int i = -1; i < 2; i++) {
				for (int j = -1; j < 2; j++) {
					class_2338 newPos = pos.method_10069(j, 0, i);
					if (shouldBreak(playerIn, worldIn, pos, newPos)) {
						targetBlocks.add(newPos);
					}
				}
			}
		}
		return targetBlocks;
	}

	private void breakBlock(class_2338 pos, class_1937 world, class_1657 playerIn, class_1799 drill) {
		class_2680 blockState = world.method_8320(pos);

		ItemPowerManager capEnergy = new ItemPowerManager(drill);

		if(capEnergy.getEnergyStored() > cost){
			capEnergy.useEnergy(cost, false);
			ExternalPowerSystems.requestEnergyFromArmor(capEnergy, playerIn);

			blockState.method_11614().method_9536(blockState, world, pos, blockState, true);
			blockState.method_11614().method_9556(world, playerIn, pos, blockState, world.method_8321(pos), drill);
			world.method_8501(pos, class_2246.field_10124.method_9564());
			world.method_8544(pos);
		}
	}
	
	private boolean shouldBreak(class_1657 playerIn, class_1937 worldIn, class_2338 originalPos, class_2338 pos) {
		if (originalPos.equals(pos)) {
			return false;
		}
		class_2680 blockState = worldIn.method_8320(pos);
		if (blockState.method_11620() == class_3614.field_15959) {
			return false;
		}
		if (blockState.method_11620().method_15797()) {
			return false;
		}
		float blockHardness = blockState.method_11589(playerIn, worldIn, pos);
		if (blockHardness == -1.0F) {
			return false;
		}
		float originalHardness = worldIn.method_8320(originalPos).method_11579(worldIn, originalPos);
		if ((originalHardness / blockHardness) > 10.0F) {
			return false;
		}
		
		return true;	
	}

	// ItemDrill
	@Override
	public boolean method_7879(class_1799 stack, class_1937 worldIn, class_2680 blockIn, class_2338 pos, class_1309 entityLiving) {
		class_1657 playerIn = null;
		if ((entityLiving instanceof class_1657)) {
			playerIn = (class_1657) entityLiving;
		}
		if(ItemUtils.isActive(stack)){
			for (class_2338 additionalPos : getTargetBlocks(worldIn, pos, playerIn)) {
				breakBlock(additionalPos, worldIn, playerIn, stack);
			}
		}
		return super.method_7879(stack, worldIn, blockIn, pos, entityLiving);
	}

	@Override
	public class_1271<class_1799> method_7836(final class_1937 world, final class_1657 player, final class_1268 hand) {
		final class_1799 stack = player.method_5998(hand);
		if (player.method_5715()) {
			if (new ItemPowerManager(stack).getEnergyStored() < cost) {
				ChatUtils.sendNoSpamMessages(MessageIDs.nanosaberID, new class_2585(
					class_124.field_1080 + class_1074.method_4662("techreborn.message.nanosaberEnergyErrorTo") + " "
						+ class_124.field_1065 + class_1074
						.method_4662("techreborn.message.nanosaberActivate")));
			} else {
				if (!ItemUtils.isActive(stack)) {
					if (stack.method_7969() == null) {
						stack.method_7980(new class_2487());
					}
					stack.method_7969().method_10556("isActive", true);
					if (world.field_9236) {
						ChatUtils.sendNoSpamMessages(MessageIDs.nanosaberID, new class_2585(
							class_124.field_1080 + class_1074.method_4662("techreborn.message.setTo") + " "
								+ class_124.field_1065 + class_1074
								.method_4662("techreborn.message.nanosaberActive")));
					}
				} else {
					stack.method_7969().method_10556("isActive", false);
					if (world.field_9236) {
						ChatUtils.sendNoSpamMessages(MessageIDs.nanosaberID, new class_2585(
							class_124.field_1080 + class_1074.method_4662("techreborn.message.setTo") + " "
								+ class_124.field_1065 + class_1074
								.method_4662("techreborn.message.nanosaberInactive")));
					}
				}
			}
			return new class_1271<>(class_1269.field_5812, stack);
		}
		return new class_1271<>(class_1269.field_5811, stack);
	}

	@Override
	public void method_7852(class_1937 world, class_1309 entity,  class_1799 stack, int i) {
		if (ItemUtils.isActive(stack) && new ItemPowerManager(stack).getEnergyStored() < cost) {
			if(entity.field_6002.field_9236){
				ChatUtils.sendNoSpamMessages(MessageIDs.nanosaberID, new class_2585(
					class_124.field_1080 + class_1074.method_4662("techreborn.message.nanosaberEnergyError") + " "
						+ class_124.field_1065 + class_1074
						.method_4662("techreborn.message.nanosaberDeactivating")));
			}
			stack.method_7969().method_10556("isActive", false);
		}
	}

	@Environment(EnvType.CLIENT)
	@Override
	public void method_7851(class_1799 stack, @Nullable class_1937 worldIn, List<class_2561> tooltip, class_1836 flagIn) {
		if (!ItemUtils.isActive(stack)) {
			tooltip.add(new class_2585(class_124.field_1054 + "Shear: " + class_124.field_1061 + class_1074.method_4662("techreborn.message.nanosaberInactive")));
		} else {
			tooltip.add(new class_2585(class_124.field_1054 + "Shear: " + class_124.field_1060 + class_1074.method_4662("techreborn.message.nanosaberActive")));
		}
	}

	// ItemPickaxe
	@Override
	public boolean method_7856(class_2680 blockIn) {
		return (class_1802.field_8377.method_7856(blockIn) || class_1802.field_8250.method_7856(blockIn)) && !class_1802.field_8556.method_7856(blockIn);
	}

	// Item
	@Environment(EnvType.CLIENT)
	@Override
	public void method_7850(class_1761 par2ItemGroup, class_2371<class_1799> itemList) {
		if (!method_7877(par2ItemGroup)) {
			return;
		}
		InitUtils.initPoweredItems(TRContent.INDUSTRIAL_DRILL, itemList);
	}
}
