/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.world;

import net.fabricmc.fabric.api.event.registry.RegistryEntryAddedCallback;
import net.minecraft.class_1959;
import net.minecraft.class_1959.class_1961;
import net.minecraft.class_2246;
import net.minecraft.class_2378;
import net.minecraft.class_2893;
import net.minecraft.class_2997;
import net.minecraft.class_3031;
import net.minecraft.class_3037;
import net.minecraft.class_3111;
import net.minecraft.class_3124;
import net.minecraft.class_3124.class_3125;
import net.minecraft.class_3276;
import net.minecraft.class_3284;
import reborncore.common.world.CustomOreFeature;
import reborncore.common.world.CustomOreFeatureConfig;
import techreborn.init.TRContent;

import java.util.ArrayList;
import java.util.List;

/**
 * @author drcrazy
 *
 */
public class WorldGenerator {

//	public static final Predicate<IBlockState> IS_ENDSTONE = (state) -> {
//		return state != null && (state.getBlock() == Blocks.END_STONE);
//	};

	private static List<class_1959> checkedBiomes = new ArrayList<>();

	public static void initBiomeFeatures() {
		for (class_1959 biome : class_2378.field_11153) {
			addToBiome(biome);
		}

		//Handles modded biomes
		RegistryEntryAddedCallback.event(class_2378.field_11153).register((i, identifier, biome) -> addToBiome(biome));
	}

	private static void addToBiome(class_1959 biome){
		if(checkedBiomes.contains(biome)){
			//Just to be sure we dont add the stuff twice to the same biome
			return;
		}
		checkedBiomes.add(biome);

		if (biome.method_8688() == class_1961.field_9366) {
			addOre(biome, class_3124.class_3125.field_13727, TRContent.Ores.CINNABAR);
			addOre(biome, class_3124.class_3125.field_13727, TRContent.Ores.PYRITE);
			addOre(biome, class_3124.class_3125.field_13727, TRContent.Ores.SPHALERITE);

		} else if (biome.method_8688() == class_1961.field_9360) {
			addEndOre(biome, TRContent.Ores.PERIDOT);
			addEndOre(biome, TRContent.Ores.SHELDONITE);
			addEndOre(biome, TRContent.Ores.SODALITE);
			addEndOre(biome, TRContent.Ores.TUNGSTEN);
		} else {
			addOre(biome, class_3124.class_3125.field_13730, TRContent.Ores.BAUXITE);
			addOre(biome, class_3124.class_3125.field_13730, TRContent.Ores.COPPER);
			addOre(biome, class_3124.class_3125.field_13730, TRContent.Ores.GALENA);
			addOre(biome, class_3124.class_3125.field_13730, TRContent.Ores.IRIDIUM);
			addOre(biome, class_3124.class_3125.field_13730, TRContent.Ores.LEAD);
			addOre(biome, class_3124.class_3125.field_13730, TRContent.Ores.RUBY);
			addOre(biome, class_3124.class_3125.field_13730, TRContent.Ores.SAPPHIRE);
			addOre(biome, class_3124.class_3125.field_13730, TRContent.Ores.SILVER);
			addOre(biome, class_3124.class_3125.field_13730, TRContent.Ores.TIN);

			if (biome.method_8688() == class_1961.field_9370 || biome.method_8688() == class_1961.field_9361) {
				biome.method_8719(class_2893.class_2895.field_13178,
				                 class_1959.method_8699(new RubberTreeFeature(class_3111::method_13565, false),
				                                        class_3037.field_13603, class_3284.field_14267,
				                                        new class_3276(1, 0.1F, 1)));
			}
		}
	}

	private static void addOre(class_1959 biome, class_3125 canReplaceIn, TRContent.Ores ore) {
		biome.method_8719(class_2893.class_2895.field_13176, class_1959.method_8699(class_3031.field_13517,
				new class_3124(canReplaceIn, ore.block.method_9564(), ore.veinSize), class_3284.field_14241,
				new class_2997(ore.veinsPerChunk, ore.minY, ore.minY, ore.maxY)));
	}

	private static void addEndOre(class_1959 biome, TRContent.Ores ore) {
		biome.method_8719(class_2893.class_2895.field_13176, class_1959.method_8699(CustomOreFeature.CUSTOM_ORE_FEATURE,
		                                                                                 new CustomOreFeatureConfig(blockState -> blockState.method_11614() == class_2246.field_10471, ore.block.method_9564(), ore.veinSize), class_3284.field_14241,
		                                                                                 new class_2997(ore.veinsPerChunk, ore.minY, ore.minY, ore.maxY)));
	}
}